import maxflow.MaxFlow;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.DefaultGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceDGS;

import java.io.*;
import java.util.concurrent.atomic.AtomicReference;

public class Autoroutes {

    private static Node source; // Noeud source
    private static Node sink; // Noeud de destination

    public static void main(String[] args) {

        System.setProperty("org.graphstream.ui", "swing"); // Configuration pour l'affichage du graphe

        // Création du graphe
        String filePath = "resources/autoroute.dgs";
        Graph g = new DefaultGraph("autoroutes");
        FileSource fs = new FileSourceDGS();

        fs.addSink(g);

        try {
            fs.readAll(filePath);
        } catch( IOException e) {
            System.err.println("Error while generating graph");
            e.printStackTrace();
            System.exit(1);
        } finally {
            fs.removeSink(g);
        }
        // Fin création du graphe

        // Affectation source et sink
        source = g.getNode("A");
        sink = g.getNode("I");

        // Calcul de l'arête à augmenter pour améliorer le flot
        AtomicReference<Edge> improveEdge = new AtomicReference<>();
        AtomicReference<Integer> flowToSink = new AtomicReference<>(0);
        g.edges().forEach((Edge e) -> { // Pour chaque arête du graphe
            int cap = (int) e.getAttribute("cap"); // On stocke la capacité
            // On augmente la capacité à 2500 (valeur arbitraire basée sur le sujet scané)
            e.setAttribute("cap", 2500);
            MaxFlow maxFlow = new MaxFlow(); // On reconfigure un maxflow
            maxFlow.setCapacityAttribute("cap");
            maxFlow.init(g);
            maxFlow.setSource(source);
            maxFlow.setSink(sink);
            maxFlow.compute();
            // On calcule le flot arrivant au sink
            Double tempFlowToSink = sink.enteringEdges().reduce(0.0, (acc, entering) -> {
                return acc + maxFlow.getFlow(entering);
            }, Double::sum);
            // Si le flot est plus élevé que précédemment calculé, alors on stocke l'arête augmentée
            // et la valeur du flot entrant dans la destination
            if(tempFlowToSink > flowToSink.get()) {
                improveEdge.set(e);
                flowToSink.set(tempFlowToSink.intValue());
            }
            // On n'oublie pa de remettre la capacité initiale de l'arête pour les prochains calculs
            e.setAttribute("cap", cap);
        });
        System.out.println("Best edge to improve : " + improveEdge.get());
        System.out.println("Flow arriving to the sink : " + flowToSink.get());

        improveEdge.get().setAttribute("cap", 2500);

        MaxFlow mf = new MaxFlow();
        mf.setCapacityAttribute("cap");
        mf.init(g);
        mf.setSource(source);
        mf.setSink(sink);
        mf.compute();
        setStyle(g, mf);
        g.display(false);
    }

    private static void setStyle(Graph g, MaxFlow mf) {
        g.edges().forEach((Edge e) -> {
            e.setAttribute("ui.style", "text-size: 15; text-alignment: along;");
            double flow = mf.getFlow(e);
            double cap = mf.getCapacity(e);
            e.setAttribute("ui.label", String.format("%.0f/%.0f", flow, cap));
            if (cap == flow) e.setAttribute("ui.style", "text-size: 15; text-alignment: along; fill-color: red;");
        });

        g.nodes().forEach((Node n) -> {
            n.setAttribute("ui.label", n.toString());
            n.setAttribute("ui.style", "fill-color: white;"
                    + "stroke-mode: plain;"
                    + "stroke-width: 2;"
                    + "stroke-color: black;"
                    + "size: "+ "24, 24;");
        });
    }
}
